/*****************************************************************************
 * common.h: common definitions
 * Collection of useful common types and macros definitions
 *****************************************************************************
 * Copyright (C) 1998, 1999, 2000 VideoLAN
 * $Id: common.h,v 1.103 2002/05/03 20:49:30 sam Exp $
 *
 * Authors: Samuel Hocevar <sam@via.ecp.fr>
 *          Vincent Seguin <seguin@via.ecp.fr>
 *          Gildas Bazin <gbazin@netcourrier.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

/*****************************************************************************
 * Required system headers
 *****************************************************************************/
#ifdef HAVE_STRING_H
#   include <string.h>                                         /* strerror() */
#endif

#ifdef HAVE_SYS_TYPES_H
#   include <sys/types.h>
#endif

/*****************************************************************************
 * Basic types definitions
 *****************************************************************************/

typedef unsigned char       u8;
typedef signed char         s8;

typedef unsigned short      u16;
typedef signed short        s16;

typedef unsigned int        u32;
typedef signed int          s32;

#if defined( _MSC_VER ) || ( defined( WIN32 ) && !defined( __MINGW32__ ) )
typedef unsigned __int64    u64;
typedef signed __int64      s64;
#else
typedef unsigned long long  u64;
typedef signed long long    s64;
#endif

typedef u8                  byte_t;

/* Boolean type */
#ifdef BOOLEAN_T_IN_SYS_TYPES_H
    /* <sys/types.h> already included */
#elif defined(BOOLEAN_T_IN_PTHREAD_H)
#   include <pthread.h>
#elif defined(BOOLEAN_T_IN_CTHREADS_H)
#   include <cthreads.h>
#else
typedef int                 boolean_t;
#endif
#ifdef SYS_GNU
#   define _MACH_I386_BOOLEAN_H_
#endif

/* ptrdiff_t definition */
#ifdef HAVE_STDDEF_H
#   include <stddef.h>
#else
#   include <malloc.h>
#   ifndef _PTRDIFF_T
#       define _PTRDIFF_T
/* Not portable in a 64-bit environment. */
typedef int                 ptrdiff_t;
#   endif
#endif

#if defined( WIN32 )
typedef int                 ssize_t;
#endif

/* Counter for statistics and profiling */
typedef unsigned long       count_t;

/* DCT elements types */
typedef s16                 dctelem_t;

/* Video buffer types */
typedef u8                  yuv_data_t;

/*****************************************************************************
 * mtime_t: high precision date or time interval
 *****************************************************************************
 * Store an high precision date or time interval. The maximum precision is the
 * micro-second, and a 64 bits integer is used to avoid any overflow (maximum
 * time interval is then 292271 years, which should be long enough for any
 * video). Date are stored as a time interval since a common date.
 * Note that date and time intervals can be manipulated using regular
 * arithmetic operators, and that no special functions are required.
 *****************************************************************************/
typedef s64 mtime_t;

/*****************************************************************************
 * Classes declaration
 *****************************************************************************/
#define DECLARE_STRUCT( name ) \
    struct name##_s;         \
    typedef struct name##_s name##_t;

/* System */
DECLARE_STRUCT(vlc)
DECLARE_STRUCT(vlc_object)
DECLARE_STRUCT(main_sys)

DECLARE_STRUCT(input_bank)
DECLARE_STRUCT(aout_bank)
DECLARE_STRUCT(vout_bank)

/* Messages */
DECLARE_STRUCT(msg_bank)
DECLARE_STRUCT(msg_subscription)

/* Plugins */
DECLARE_STRUCT(plugin_bank)
DECLARE_STRUCT(plugin_info)

/* Playlist */
DECLARE_STRUCT(playlist)
DECLARE_STRUCT(playlist_item)

/* Modules */
DECLARE_STRUCT(module_bank)
DECLARE_STRUCT(module)
DECLARE_STRUCT(module_config)
DECLARE_STRUCT(module_symbols)
DECLARE_STRUCT(module_functions)

/* Interface */
DECLARE_STRUCT(intf_thread)
DECLARE_STRUCT(intf_sys)
DECLARE_STRUCT(intf_console)
DECLARE_STRUCT(intf_msg)
DECLARE_STRUCT(intf_channel)

/* Input */
DECLARE_STRUCT(input_thread)
DECLARE_STRUCT(input_channel)
DECLARE_STRUCT(input_cfg)
DECLARE_STRUCT(input_area)
DECLARE_STRUCT(input_buffers)

/* Audio */
DECLARE_STRUCT(aout_thread)
DECLARE_STRUCT(aout_sys)
DECLARE_STRUCT(aout_fifo)

/* Video */
DECLARE_STRUCT(vout_thread)
DECLARE_STRUCT(vout_font)
DECLARE_STRUCT(vout_sys)
DECLARE_STRUCT(chroma_sys)
DECLARE_STRUCT(video_parser)
DECLARE_STRUCT(picture)
DECLARE_STRUCT(picture_sys)
DECLARE_STRUCT(picture_heap)
DECLARE_STRUCT(subpicture)
DECLARE_STRUCT(subpicture_sys)

/* Decoders */
DECLARE_STRUCT(decoder_config)
DECLARE_STRUCT(decoder_fifo)

/* Misc */
DECLARE_STRUCT(macroblock)
DECLARE_STRUCT(data_packet)
DECLARE_STRUCT(data_buffer)
DECLARE_STRUCT(downmix)
DECLARE_STRUCT(imdct)
DECLARE_STRUCT(complex)
DECLARE_STRUCT(dm_par)
DECLARE_STRUCT(es_descriptor)
DECLARE_STRUCT(pgrm_descriptor)
DECLARE_STRUCT(stream_descriptor)
DECLARE_STRUCT(stream_ctrl)
DECLARE_STRUCT(pes_packet)
DECLARE_STRUCT(bit_stream)
DECLARE_STRUCT(network_socket)

/*****************************************************************************
 * OS-specific headers and thread types
 *****************************************************************************/
#include "os_specific.h"
#include "vlc_threads.h"

/*****************************************************************************
 * Compiler-specific workarounds
 *****************************************************************************/
#if defined( __BORLANDC__ )
#   undef HAVE_VARIADIC_MACROS
#endif

/*****************************************************************************
 * Common structure members
 *****************************************************************************/

/* VLC_COMMON_MEMBERS : members common to all basic vlc objects */
#define VLC_COMMON_MEMBERS                                                    \
    int i_pid;                                                                \
    char *psz_type;                                                           \
                                                                              \
    boolean_t b_die;                                                          \
    boolean_t b_error;                                                        \
    boolean_t b_active;                                                       \
                                                                              \
    vlc_object_t *  p_this;                      /* a pointer to ourselves */ \
    vlc_t *         p_vlc;                             /* root of all evil */ \
                                                                              \
    vlc_object_t ** pp_parents;                             /* our parents */ \
    int             i_parents;                                                \
    vlc_object_t ** pp_children;                           /* our children */ \
    int             i_children;                                               \
                                                                              \
    /* Just a reminder so that people don't cast garbage */                   \
    int be_sure_to_add_VLC_COMMON_MEMBERS_to_struct;                          \

/* The real vlc_object_t type. Yes, it's that simple :-) */
struct vlc_object_s
{
    VLC_COMMON_MEMBERS
};

/* CAST_TO_VLC_OBJECT: attempt at doing a clever cast */
#define CAST_TO_VLC_OBJECT( x ) \
    ((vlc_object_t *)(x))+0*(x)->be_sure_to_add_VLC_COMMON_MEMBERS_to_struct

/*****************************************************************************
 * Macros and inline functions
 *****************************************************************************/
#ifdef NTOHL_IN_SYS_PARAM_H
#   include <sys/param.h>

#elif defined(WIN32)
/* Swap bytes in 16 bit value.  */
#   define __bswap_constant_16(x) \
     ((((x) >> 8) & 0xff) | (((x) & 0xff) << 8))

#   if defined __GNUC__ && __GNUC__ >= 2
#       define __bswap_16(x) \
            (__extension__                                                    \
             ({ register unsigned short int __v;                              \
                if (__builtin_constant_p (x))                                 \
                  __v = __bswap_constant_16 (x);                              \
                else                                                          \
                  __asm__ __volatile__ ("rorw $8, %w0"                        \
                                        : "=r" (__v)                          \
                                        : "0" ((unsigned short int) (x))      \
                                        : "cc");                              \
                __v; }))
#   else
/* This is better than nothing.  */
#       define __bswap_16(x) __bswap_constant_16 (x)
#   endif

/* Swap bytes in 32 bit value.  */
#   define __bswap_constant_32(x) \
        ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) |            \
         (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))

#   if defined __GNUC__ && __GNUC__ >= 2
/* To swap the bytes in a word the i486 processors and up provide the
   `bswap' opcode.  On i386 we have to use three instructions.  */
#       if !defined __i486__ && !defined __pentium__ && !defined __pentiumpro__
#           define __bswap_32(x) \
                (__extension__                                                \
                 ({ register unsigned int __v;                                \
                    if (__builtin_constant_p (x))                             \
                      __v = __bswap_constant_32 (x);                          \
                    else                                                      \
                      __asm__ __volatile__ ("rorw $8, %w0;"                   \
                                            "rorl $16, %0;"                   \
                                            "rorw $8, %w0"                    \
                                            : "=r" (__v)                      \
                                            : "0" ((unsigned int) (x))        \
                                            : "cc");                          \
                    __v; }))
#       else
#           define __bswap_32(x) \
                (__extension__                                                \
                 ({ register unsigned int __v;                                \
                    if (__builtin_constant_p (x))                             \
                      __v = __bswap_constant_32 (x);                          \
                    else                                                      \
                      __asm__ __volatile__ ("bswap %0"                        \
                                            : "=r" (__v)                      \
                                            : "0" ((unsigned int) (x)));      \
                    __v; }))
#       endif
#   else
#       define __bswap_32(x) __bswap_constant_32 (x)
#   endif

#   if defined __GNUC__ && __GNUC__ >= 2
/* Swap bytes in 64 bit value.  */
#       define __bswap_constant_64(x) \
            ((((x) & 0xff00000000000000ull) >> 56)                            \
             | (((x) & 0x00ff000000000000ull) >> 40)                          \
             | (((x) & 0x0000ff0000000000ull) >> 24)                          \
             | (((x) & 0x000000ff00000000ull) >> 8)                           \
             | (((x) & 0x00000000ff000000ull) << 8)                           \
             | (((x) & 0x0000000000ff0000ull) << 24)                          \
             | (((x) & 0x000000000000ff00ull) << 40)                          \
             | (((x) & 0x00000000000000ffull) << 56))

#       define __bswap_64(x) \
            (__extension__                                                    \
             ({ union { __extension__ unsigned long long int __ll;            \
                        unsigned long int __l[2]; } __w, __r;                 \
                if (__builtin_constant_p (x))                                 \
                  __r.__ll = __bswap_constant_64 (x);                         \
                else                                                          \
                  {                                                           \
                    __w.__ll = (x);                                           \
                    __r.__l[0] = __bswap_32 (__w.__l[1]);                     \
                    __r.__l[1] = __bswap_32 (__w.__l[0]);                     \
                  }                                                           \
                __r.__ll; }))
#   else
#       define __bswap_64(i) \
            (u64)((__bswap_32((i) & 0xffffffff) << 32) |                      \
            __bswap_32(((i) >> 32) & 0xffffffff ))
#   endif

#else /* NTOHL_IN_SYS_PARAM_H || WIN32 */
#   include <netinet/in.h>

#endif /* NTOHL_IN_SYS_PARAM_H || WIN32 */

/* CEIL: division with round to nearest greater integer */
#define CEIL(n, d)  ( ((n) / (d)) + ( ((n) % (d)) ? 1 : 0) )

/* PAD: PAD(n, d) = CEIL(n ,d) * d */
#define PAD(n, d)   ( ((n) % (d)) ? ((((n) / (d)) + 1) * (d)) : (n) )

/* __MAX and __MIN: self explanatory */
#ifndef __MAX
#   define __MAX(a, b)   ( ((a) > (b)) ? (a) : (b) )
#endif
#ifndef __MIN
#   define __MIN(a, b)   ( ((a) < (b)) ? (a) : (b) )
#endif

/* MSB (big endian)/LSB (little endian) conversions - network order is always
 * MSB, and should be used for both network communications and files. Note that
 * byte orders other than little and big endians are not supported, but only
 * the VAX seems to have such exotic properties - note that these 'functions'
 * needs <netinet/in.h> or the local equivalent. */
#if !defined( WIN32 )
#if WORDS_BIGENDIAN
#   define hton16      htons
#   define hton32      htonl
#   define hton64(i)   ( i )
#   define ntoh16      ntohs
#   define ntoh32      ntohl
#   define ntoh64(i)   ( i )
#else
#   define hton16      htons
#   define hton32      htonl
    static inline u64 __hton64( u64 i )
    {
        return ((u64)(htonl((i) & 0xffffffff)) << 32)
                | htonl(((i) >> 32) & 0xffffffff );
    }
#   define hton64(i)   __hton64( i )
#   define ntoh16      ntohs
#   define ntoh32      ntohl
#   define ntoh64      hton64
#endif
#endif /* !defined( WIN32 ) */

/* Macros with automatic casts */
#define U64_AT(p)   ( ntoh64 ( *( (u64 *)(p) ) ) )
#define U32_AT(p)   ( ntoh32 ( *( (u32 *)(p) ) ) )
#define U16_AT(p)   ( ntoh16 ( *( (u16 *)(p) ) ) )

/* Alignment of critical static data structures */
#ifdef ATTRIBUTE_ALIGNED_MAX
#   define ATTR_ALIGN(align) __attribute__ ((__aligned__ ((ATTRIBUTE_ALIGNED_MAX < align) ? ATTRIBUTE_ALIGNED_MAX : align)))
#else
#   define ATTR_ALIGN(align)
#endif

/* Alignment of critical dynamic data structure
 *
 * Not all platforms support memalign so we provide a vlc_memalign wrapper
 * void *vlc_memalign( size_t align, size_t size, void **pp_orig )
 * *pp_orig is the pointer that has to be freed afterwards.
 */
#if 0
#ifdef HAVE_POSIX_MEMALIGN
#   define vlc_memalign(align,size,pp_orig) \
    ( !posix_memalign( pp_orig, align, size ) ? *(pp_orig) : NULL )
#endif
#endif
#ifdef HAVE_MEMALIGN
    /* Some systems have memalign() but no declaration for it */
    void * memalign( size_t align, size_t size );

#   define vlc_memalign(pp_orig,align,size) \
    ( *(pp_orig) = memalign( align, size ) )

#else /* We don't have any choice but to align manually */
#   define vlc_memalign(pp_orig,align,size) \
    (( *(pp_orig) = malloc( size + align - 1 )) \
        ? (void *)( (((unsigned long)*(pp_orig)) + (unsigned long)(align-1) ) \
                       & (~(unsigned long)(align-1)) ) \
        : NULL )

#endif


#define I64C(x)         x##LL


#if defined( WIN32 )
/* The ntoh* and hton* bytes swapping functions are provided by winsock
 * but for conveniency and speed reasons it is better to implement them
 * ourselves. ( several plugins use them and it is too much hassle to link
 * winsock with each of them ;-)
 */
#   ifdef WORDS_BIGENDIAN
#       define ntoh32(x)       (x)
#       define ntoh16(x)       (x)
#       define ntoh64(x)       (x)
#       define hton32(x)       (x)
#       define hton16(x)       (x)
#       define hton64(x)       (x)
#   else
#       define ntoh32(x)     __bswap_32 (x)
#       define ntoh16(x)     __bswap_16 (x)
#       define ntoh64(x)     __bswap_32 (x)
#       define hton32(x)     __bswap_32 (x)
#       define hton16(x)     __bswap_16 (x)
#       define hton64(x)     __bswap_64 (x)
#   endif

/* win32, cl and icl support */
#   if defined( _MSC_VER ) || !defined( __MINGW32__ )
#       define __attribute__(x)
#       define __inline__      __inline
#       define strncasecmp     strnicmp
#       define strcasecmp      stricmp
#       define S_IFBLK         0x3000  /* Block */
#       define S_ISBLK(m)      (0)
#       define S_ISCHR(m)      (0)
#       define S_ISFIFO(m)     (((m)&_S_IFMT) == _S_IFIFO)
#       define S_ISREG(m)      (((m)&_S_IFMT) == _S_IFREG)
#       undef I64C
#       define I64C(x)         x##i64
#   endif

/* several type definitions */
#   if defined( __MINGW32__ )
#       if !defined( _OFF_T_ )
typedef long long _off_t;
typedef _off_t off_t;
#           define _OFF_T_
#       else
#           define off_t long long
#       endif
#   endif

#   if defined( _MSC_VER )
#       if !defined( _OFF_T_DEFINED )
typedef __int64 off_t;
#           define _OFF_T_DEFINED
#       else
#           define off_t __int64
#       endif
#   endif

#   if defined( __BORLANDC__ )
#       undef off_t
#       define off_t unsigned __int64
#   endif

#   ifndef O_NONBLOCK
#       define O_NONBLOCK 0
#   endif

#   ifndef snprintf
#       define snprintf _snprintf  /* snprintf not defined in mingw32 (bug?) */
#   endif

#endif

/*****************************************************************************
 * CPU capabilities
 *****************************************************************************/
#define CPU_CAPABILITY_NONE    0
#define CPU_CAPABILITY_486     (1<<0)
#define CPU_CAPABILITY_586     (1<<1)
#define CPU_CAPABILITY_PPRO    (1<<2)
#define CPU_CAPABILITY_MMX     (1<<3)
#define CPU_CAPABILITY_3DNOW   (1<<4)
#define CPU_CAPABILITY_MMXEXT  (1<<5)
#define CPU_CAPABILITY_SSE     (1<<6)
#define CPU_CAPABILITY_ALTIVEC (1<<16)
#define CPU_CAPABILITY_FPU     (1<<31)

/*****************************************************************************
 * I18n stuff
 *****************************************************************************/
#ifndef PACKAGE /* Borland C++ uses this ! */
#define PACKAGE VLC_PACKAGE
#endif
#define VERSION VLC_VERSION

#if defined( ENABLE_NLS ) && defined ( HAVE_GETTEXT ) && !defined( __BORLANDC__ ) && !defined( MODULE_NAME_IS_gnome )
#   include <libintl.h>
#   undef _
#   define _(String) dgettext (PACKAGE, String)
#   ifdef gettext_noop
#       define N_(String) gettext_noop (String)
#   else
#       define N_(String) (String)
#   endif
#elif !defined( MODULE_NAME_IS_gnome )
#   define _(String) (String)
#   define N_(String) (String)
#endif

/*****************************************************************************
 * Plug-in stuff
 *****************************************************************************/
struct module_symbols_s
{
    int  ( *   vlc_threads_init )  ( vlc_object_t * );
    int  ( *   vlc_threads_end )   ( void );
    int  ( *   vlc_mutex_init )    ( vlc_object_t *, vlc_mutex_t * );
    int  ( * __vlc_mutex_destroy ) ( char *, int, vlc_mutex_t * );
    int  ( *   vlc_cond_init )     ( vlc_cond_t * );
    int  ( * __vlc_cond_destroy )  ( char *, int, vlc_cond_t * );
    int  ( * __vlc_thread_create ) ( vlc_object_t *, char *, int,
                           vlc_thread_t *, char *, vlc_thread_func_t, void * );
    void ( *   vlc_thread_exit )   ( void );
    void ( * __vlc_thread_join )   ( char *, int, vlc_thread_t );

    int    ( * config_GetInt )  ( vlc_object_t *, const char * );
    float  ( * config_GetFloat )( vlc_object_t *, const char * );
    char * ( * config_GetPsz )  ( vlc_object_t *, const char * );
    void   ( * config_PutInt )  ( vlc_object_t *, const char *, int );
    void   ( * config_PutFloat )( vlc_object_t *, const char *, float );
    void   ( * config_PutPsz )  ( vlc_object_t *, const char *, char* );
    int    ( * config_LoadConfigFile )  ( vlc_object_t *, const char * );
    int    ( * config_SaveConfigFile )  ( vlc_object_t *, const char * );
    module_config_t * ( * config_FindConfig ) ( vlc_object_t *, const char * );
    module_config_t * ( * config_Duplicate ) ( module_config_t * );
    void   ( * config_SetCallbacks )  ( module_config_t *, module_config_t * );
    void   ( * config_UnsetCallbacks )( module_config_t * );

    void ( * __msg_Generic )     ( vlc_object_t *, int, const char *,
                                   const char *, ... );
    void ( * __msg_Info )        ( void *, const char *, ... );
    void ( * __msg_Warn )        ( void *, const char *, ... );
    void ( * __msg_Err )         ( void *, const char *, ... );

    msg_subscription_t * ( * msg_Subscribe ) ( vlc_object_t * );
    void ( * msg_Unsubscribe )   ( vlc_object_t *, msg_subscription_t * );

    int  ( * intf_PlaylistAdd )     ( playlist_t *, int, const char * );
    int  ( * intf_PlaylistDelete )  ( playlist_t *, int );
    void ( * intf_PlaylistNext )    ( playlist_t * );
    void ( * intf_PlaylistPrev )    ( playlist_t * );
    void ( * intf_PlaylistDestroy ) ( playlist_t * );
    void ( * intf_PlaylistJumpto )  ( playlist_t *, int );

    void ( * intf_UrlDecode )       ( char * );
    int  ( * intf_Eject )           ( vlc_object_t *, const char * );

    void    ( * msleep )            ( mtime_t );
    mtime_t ( * mdate )             ( void );
    char  * ( * mstrtime )          ( char *, mtime_t );

    int  ( * network_ChannelCreate )( vlc_object_t * );
    int  ( * network_ChannelJoin )  ( vlc_object_t *, int );

    int  ( * input_SetProgram )     ( input_thread_t *, pgrm_descriptor_t * );
    void ( * input_SetStatus )      ( input_thread_t *, int );
    void ( * input_Seek )           ( input_thread_t *, off_t );
    void ( * input_DumpStream )     ( input_thread_t * );
    char * ( * input_OffsetToTime ) ( input_thread_t *, char *, off_t );
    int  ( * input_ChangeES )       ( input_thread_t *,
                                      es_descriptor_t *, u8 );
    int  ( * input_ToggleES )       ( input_thread_t *,
                                      es_descriptor_t *, boolean_t );
    int  ( * input_ChangeArea )     ( input_thread_t *, input_area_t * );
    int  ( * input_ChangeProgram )  ( input_thread_t *, u16 );
    es_descriptor_t * ( * input_FindES ) ( input_thread_t *, u16 );
    pgrm_descriptor_t * ( * input_FindProgram ) ( input_thread_t *, u16 );
    es_descriptor_t * ( * input_AddES ) ( input_thread_t *,
                                          pgrm_descriptor_t *, u16, size_t );
    void ( * input_DelES )          ( input_thread_t *, es_descriptor_t * );
    int  ( * input_SelectES )       ( input_thread_t *, es_descriptor_t * );
    int  ( * input_UnselectES )     ( input_thread_t *, es_descriptor_t * );
    pgrm_descriptor_t * ( * input_AddProgram ) ( input_thread_t *,
                                                 u16, size_t );
    void ( * input_DelProgram )     ( input_thread_t *, pgrm_descriptor_t * );
    input_area_t * ( * input_AddArea ) ( input_thread_t * );
    void ( * input_DelArea )        ( input_thread_t *, input_area_t * );

    void ( * InitBitstream )        ( bit_stream_t *,
                                      decoder_fifo_t *,
                                      void ( * ) ( bit_stream_t *,
                                                   boolean_t ),
                                      void * );
    void ( * BitstreamNextDataPacket )( bit_stream_t * );
    boolean_t ( * NextDataPacket )  ( decoder_fifo_t *,
                                      data_packet_t ** );
    void ( * DecoderError )         ( decoder_fifo_t * p_fifo );
    int  ( * input_InitStream )     ( input_thread_t *, size_t );
    void ( * input_EndStream )      ( input_thread_t * );

    void ( * input_ParsePES )       ( input_thread_t *, es_descriptor_t * );
    void ( * input_GatherPES )      ( input_thread_t *, data_packet_t *,
                                      es_descriptor_t *,
                                      boolean_t, boolean_t );
    void ( * input_DecodePES )      ( decoder_fifo_t *, pes_packet_t * );
    es_descriptor_t * ( * input_ParsePS ) ( input_thread_t *,
                                            data_packet_t * );
    ssize_t ( * input_ReadPS )      ( input_thread_t *, data_packet_t ** );
    void ( * input_DemuxPS )        ( input_thread_t *, data_packet_t * );
    ssize_t ( * input_ReadTS )      ( input_thread_t *, data_packet_t ** );
    void ( * input_DemuxTS )        ( input_thread_t *, data_packet_t *,
                                      void(* pf_psi_callback) 
                                           ( input_thread_t *, data_packet_t *,
                                             es_descriptor_t *, boolean_t ) );
    void ( * input_ClockManageRef )     ( input_thread_t *,
                                          pgrm_descriptor_t *, mtime_t );
    int ( * input_ClockManageControl )  ( input_thread_t *,
                                          pgrm_descriptor_t *, mtime_t );
    void ( * input_FDSeek )             ( input_thread_t *, off_t );
    void ( * input_FDClose )            ( input_thread_t * );
    ssize_t ( * input_FDRead )          ( input_thread_t *, byte_t *, size_t );
    ssize_t ( * input_FDNetworkRead )   ( input_thread_t *, byte_t *, size_t );

    void * ( * input_BuffersInit )( vlc_object_t * );
    void ( * input_BuffersEnd )( input_thread_t *, input_buffers_t * );
    data_buffer_t * ( * input_NewBuffer )( input_buffers_t *, size_t );
    void ( * input_ReleaseBuffer )( input_buffers_t *, data_buffer_t * );
    data_packet_t * ( * input_ShareBuffer )( input_buffers_t *,
                                             data_buffer_t * );
    data_packet_t * ( * input_NewPacket )( input_buffers_t *, size_t );
    void ( * input_DeletePacket )( input_buffers_t *, data_packet_t * );
    pes_packet_t * ( * input_NewPES )( input_buffers_t * );
    void ( * input_DeletePES )( input_buffers_t *, pes_packet_t * );
    ssize_t ( * input_FillBuffer )( input_thread_t * );
    ssize_t ( * input_Peek )( input_thread_t *, byte_t **, size_t );
    ssize_t ( * input_SplitBuffer )( input_thread_t *, data_packet_t **, size_t );
    int ( * input_AccessInit )( input_thread_t * );
    void ( * input_AccessReinit )( input_thread_t * );
    void ( * input_AccessEnd )( input_thread_t * );

    aout_fifo_t * ( * aout_CreateFifo ) ( vlc_object_t *,
                                          int, int, int, int, void * );
    void ( * aout_DestroyFifo )     ( aout_fifo_t * );

    vout_thread_t * (* vout_CreateThread) ( vlc_object_t *,
                                            int *, int, int, u32, int );
    void  ( * vout_DestroyThread )  ( vout_thread_t *, int * );

    picture_t * ( * vout_CreatePicture ) ( vout_thread_t *,
                                           boolean_t, boolean_t, boolean_t ); 
    void  ( * vout_AllocatePicture )( vout_thread_t *, picture_t *,
                                      int, int, u32 );
    void  ( * vout_DisplayPicture ) ( vout_thread_t *, picture_t * );
    void  ( * vout_DestroyPicture ) ( vout_thread_t *, picture_t * );
    void  ( * vout_LinkPicture )    ( vout_thread_t *, picture_t * );
    void  ( * vout_UnlinkPicture )  ( vout_thread_t *, picture_t * );
    void  ( * vout_DatePicture )    ( vout_thread_t *, picture_t *, mtime_t );
    void  ( * vout_PlacePicture )   ( vout_thread_t *, int, int,
                                      int *, int *, int *, int * );
    int   ( * vout_ChromaCmp )      ( u32, u32 );

    subpicture_t * (* vout_CreateSubPicture) ( vout_thread_t *, int, int );
    void  ( * vout_DestroySubPicture )  ( vout_thread_t *, subpicture_t * );
    void  ( * vout_DisplaySubPicture )  ( vout_thread_t *, subpicture_t * );
    
    u32  ( * UnalignedShowBits )    ( bit_stream_t *, unsigned int );
    void ( * UnalignedRemoveBits )  ( bit_stream_t * );
    u32  ( * UnalignedGetBits )     ( bit_stream_t *, unsigned int );
    void ( * CurrentPTS )           ( bit_stream_t *, mtime_t *, mtime_t * );

    char * ( * DecodeLanguage ) ( u16 );

    module_t * ( * module_Need )   ( vlc_object_t *, int, char *, void * );
    void       ( * module_Unneed ) ( module_t * );
};

#ifdef __PLUGIN__
extern module_symbols_t* p_symbols;
#endif

